from functools import lru_cache
import pathlib
import subprocess

from . import _except_return_none

@lru_cache()
@_except_return_none
def get_git_revision():
    return subprocess.check_output([
        'git',
        '-C',
        pathlib.Path(__file__).parent.absolute(),
        'rev-parse',
        '--short',
        'HEAD',
    ]).decode('ascii').strip()



import ipynbname
import os

from . import _except_return_none

@_except_return_none
def get_notebook_name():
    try:
        return ipynbname.name()
    except:
        return str(
            os.getenv('NOTEBOOK_NAME')
        )



import inspect
import types

from . import _except_return_none

@_except_return_none
def get_package_versions():

    caller_globals = inspect.stack()[1][0].f_globals

    caller_base_module_names = [
        val.__name__.split('.')[0]
        for val in caller_globals.values()
        if isinstance(val, types.ModuleType)
    ] + [
        'IPython',
    ]

    res = {}

    for base_module in sorted(caller_base_module_names):
        exec(f'import {base_module}', globals())
        exec(
            f'version = getattr({base_module}, "__version__", None)',
            globals(),
        )
        if version:
            res[base_module] = version

    return res



from inspect import currentframe, getframeinfo
import re

from . import _except_return_none

@_except_return_none
def get_notebook_cell_execution_count():
    frame = currentframe()
    while not re.search('ipython-input-[0-9]+-', getframeinfo(frame).filename):
        frame = frame.f_back

    return int(re.search(
        'ipython-input-([0-9]+)-',
        getframeinfo(frame).filename,
    ).group(1))



import functools

def _except_return_none(func):

  @functools.wraps(func)
  def wrapper_except_return_none(*args, **kwargs):
    try:
      return func(*args, **kwargs)
    except:
      return None

  return wrapper_except_return_none



from datetime import datetime, timezone

from . import _except_return_none

@_except_return_none
def get_timestamp():
    return datetime.now().replace(
        tzinfo=timezone.utc,
    ).replace(
        microsecond=0,
    ).isoformat().replace('+', 'Z')



"""Top-level package for nbmetalog."""

__author__ = """Matthew Andres Moreno"""
__email__ = 'm.more500@gmail.com'
__version__ = '0.1.0'

from ._except_return_none import _except_return_none
from .get_env_context import get_env_context
from .get_git_revision import get_git_revision
from .get_hostname import get_hostname
from .get_interpreter import get_interpreter
from .get_notebook_cell_execution_count import get_notebook_cell_execution_count
from .get_notebook_name import get_notebook_name
from .get_notebook_path import get_notebook_path
from .get_package_versions import get_package_versions
from .get_session_uuid import get_session_uuid
from .get_timestamp import get_timestamp



#!/usr/bin/env python

'''
`dub` tests for `nbmetalog` package.
'''

from keyname import keyname as kn
import pytest

from nbmetalog import nbmetalog as nbm

def test_collate_summary_metadata():
    nbm.collate_summary_metadata()

def test_collate_outattr_metadata():
    for k, v in nbm.collate_outattr_metadata().items():
        first, *rest = k
        assert first == '_'
        if v is not None:
            assert kn.demote(v) == v



"""Main module."""

from keyname import keyname as kn
import yaml

from . import get_env_context
from . import get_git_revision
from . import get_hostname
from . import get_interpreter
from . import get_notebook_cell_execution_count
from . import get_notebook_name
from . import get_notebook_path
from . import get_package_versions
from . import get_session_uuid
from . import get_timestamp

def collate_summary_metadata():
    return {
        'context' : get_env_context(),
        'nbcellexec' : get_notebook_cell_execution_count(),
        'nbname' : get_notebook_name(),
        'nbpath' : get_notebook_path(),
        'revision' : get_git_revision(),
        'session' : get_session_uuid(),
        'timestamp' : get_timestamp(),
    }

def collate_outattr_metadata():
    return {
        '_' + k : kn.demote(v) if v is not None else None
        for k, v in collate_summary_metadata().items()
    }

def collate_full_metadata():
    return {
        'context' : get_env_context(),
        'hostname' : get_hostname(),
        'interpreter' : get_interpreter(),
        'nbcellexec' : get_notebook_cell_execution_count(),
        'nbname' : get_notebook_name(),
        'nbpath' : get_notebook_path(),
        'revision' : get_git_revision(),
        'session' : get_session_uuid(),
        'timestamp' : get_timestamp(),
    }


def print_metadata():

    print(
        yaml.dump( collate_full_metadata() )
    )

    print()

    for k, v in get_package_versions().items():
        print(f'{k}=={v}')
